@@ -490,6 +490,16 @@ do_open (GnomeVFSMethod *method,
 		return GNOME_VFS_ERROR_IS_DIRECTORY;
 	}
 
+	if ((card != -1) && (folder == -1) && (is_read == FALSE))
+	{
+		/* trying to create a file under rio500://internal/
+		 * like: rio500://internal/foo.mp3 <- that won't work */
+		UNLOCK_RIO();
+		DEBUG_RIO(("do_open(): tried to create a file without creating"
+					" a folder first\n"));
+		return GNOME_VFS_ERROR_NOT_PERMITTED;
+	}
+
 	if ((song == -1) && (is_read == TRUE))
 	{
 		UNLOCK_RIO();
