@@ -400,6 +400,7 @@ smb_server_connection_new_from_ip (struct in_addr *ip,
 		user_copy = g_strdup (user);
 	}
 
+	DEBUG_SMB2(("Trying to setup session as user %s\n", user_copy));
 	if (!cli_session_setup (cli,
 				/* Username: */
 				user_copy,
@@ -410,6 +411,7 @@ smb_server_connection_new_from_ip (struct in_addr *ip,
 				/* Workgroup: */
 				workgroup)) {
 		/* Try anon login */
+		DEBUG_SMB2(("Failed login as %s, trying anon\n", user_copy));
 		if (!cli_session_setup (cli,
 					"",
 					"", 0,
@@ -638,7 +640,7 @@ smb_root_connection (SmbConnection **connection)
 	}
 
 	if (!get_a_master_browser (&ip)) {
-		g_warning ("Couldn't find a master browser");
+		DEBUG_SMB2(("Couldn't find a master browser\n"));
 		return GNOME_VFS_ERROR_NO_MASTER_BROWSER;
 	}
 
@@ -664,6 +666,7 @@ smb_workgroup_connection_find (SmbURIInfo *info, gboolean can_fail, SmbConnectio
 
 	result = smb_root_connection (&root);
 	if (result != GNOME_VFS_OK) {
+		DEBUG_SMB2(("smb_workgroup_connection_find (): couldn't the root connection\n"));
 		return result;
 	}
 
@@ -677,6 +680,8 @@ smb_workgroup_connection_find (SmbURIInfo *info, gboolean can_fail, SmbConnectio
 		result = gnome_vfs_result_from_cli (root->cli);
 		smb_connection_free_if_necessary (root, result);
 
+		DEBUG_SMB2(("smb_workgroup_connection_find (): cli_NetServerEnum failed\n"));
+
 		return result;
 	}
 
@@ -696,6 +701,8 @@ smb_workgroup_connection_find (SmbURIInfo *info, gboolean can_fail, SmbConnectio
 							      FALSE, FALSE,
 							      connection);
 				g_free (fake_info);
+
+				DEBUG_SMB2(("smb_workgroup_connection_find (): smb_host_connection (user: %s password: %s) result: %s", info->user_name, info->password, gnome_vfs_result_to_string (result)));
 			}
 		} else {
 			result = GNOME_VFS_OK;
@@ -704,6 +711,8 @@ smb_workgroup_connection_find (SmbURIInfo *info, gboolean can_fail, SmbConnectio
 
 		g_free (workgroup_master_helper);
 		workgroup_master_helper = NULL;
+	} else {
+		result = GNOME_VFS_ERROR_NOT_FOUND;
 	}
 
 	return result;
@@ -825,7 +834,6 @@ smb_share_connection_new (SmbURIInfo *info,
 
 		node = g_node_find_child (connection_root, G_TRAVERSE_ALL, host);
 		g_assert (node != NULL);
-
 		g_node_append_data (node, *share);
 	}
 
@@ -945,13 +953,21 @@ get_file_type (SmbURIInfo *info, gboolean can_fail, SmbLookupData **data)
 	/* Host name or Workgroup */
 
 	type = SMB_VIRTUAL_TYPE_HOST;
-	
+
+	/* First look for already existing host
+	 * Can't always try to create one, as a failed lookup is slow */
 	result = smb_host_connection (info, TRUE, FALSE, &host);
+	DEBUG_SMB2(("get_file_type (), first smb_host_connection () said: %s\n", gnome_vfs_result_to_string (result)));
 	if (result != GNOME_VFS_OK) {
+		/* If failed, look for workgroup with that name */
 		result = smb_workgroup_connection_find (info, can_fail, &host);
 		if (result == GNOME_VFS_OK) {
 			type = (info->type == SMB_VIRTUAL_TYPE_UNKNOWN) ?
 				SMB_VIRTUAL_TYPE_WORKGROUP : info->type;
+		} else {
+			/* Last resort, look for new host */
+			result = smb_host_connection (info, FALSE, FALSE, &host);
+			DEBUG_SMB2(("get_file_type (), second smb_host_connection () said: %s\n", gnome_vfs_result_to_string (result)));
 		}
 	}
 	
@@ -988,22 +1004,11 @@ get_file_type (SmbURIInfo *info, gboolean can_fail, SmbLookupData **data)
 				g_free (*data);
 			}
 		} else if ((*data)->type == SMB_VIRTUAL_TYPE_WORKGROUP) {
-			SmbURIInfo *fake_info;
-
-			fake_info = g_new0 (SmbURIInfo, 1);
-			fake_info->host_name = info->share_name;
-			fake_info->user_name = info->user_name;
-			fake_info->password = info->password;
-
-			result = smb_host_connection (fake_info, FALSE, FALSE, &host);
-			g_free (fake_info);
-
-			if (result == GNOME_VFS_OK) {
-				(*data)->path = g_strdup (info->share_name);
-				(*data)->type = SMB_VIRTUAL_TYPE_HOST_FILE;
-				(*data)->connection = host;
-			} else
-				g_free (*data);
+			/* Just assume that the workgroup exists, making a connection
+			   here is way to slow when enumerating a workgroup. */
+			(*data)->path = g_strdup (info->share_name);
+			(*data)->type = SMB_VIRTUAL_TYPE_HOST_FILE;
+			(*data)->connection = host;
 		} else {
 			result = GNOME_VFS_ERROR_BAD_PARAMETERS;
 		}
@@ -1090,6 +1095,7 @@ lookup_uri (GnomeVFSURI *uri, SmbLookupData **data, gboolean can_fail)
 	DEBUG_SMB (("lookup_uri (uri: %s)\n", gnome_vfs_uri_to_string (uri, 0)));
 
 	info = smb_uri_info_new (uri);
+	DEBUG_SMB (("lookup_uri (user: %s password: %s host_name: %s share_name: %s path: %s)\n", info->user_name, info->password, info->host_name, info->share_name, info->path));
 	result = get_file_type (info, can_fail, data);
 	smb_uri_info_free (info);
 
@@ -1133,6 +1139,8 @@ do_open (GnomeVFSMethod *method,
 
 	if (res) {
 		UNLOCK_SAMBA ();
+		DEBUG_SMB(("do_open () returning res: %s\n",
+					gnome_vfs_result_to_string (res)));
 		return res;
 	}
 
@@ -1162,6 +1170,7 @@ do_open (GnomeVFSMethod *method,
 		fnum = cli_open (connection->cli, dos_filename, flags, DENY_NONE);
 		g_free (dos_filename);
 		if (fnum == -1) {
+			DEBUG_SMB (("do_open() cli_open failed\n"));
 			res = gnome_vfs_result_from_cli (connection->cli);
 			smb_connection_free_if_necessary (connection, res);
 			smb_lookup_data_free (data);
